;File: 				BlitzEvent_Basic.bb
;Author: 			Silver_Knee
;Description: 		Demonstriert die einfachste Anwendung der BlitzEvents.
;Original Uploaded: www.BlitzForum.de

;---{BlitzEvents v1.0}---

;---{Public Constants}---

Const BLITZEVENT_ERROR_OTHER=-1			;EventErrorID
Const BLITZEVENT_ERROR_NOTFUND=0		;EventErrorID
Const BLITZEVENT_ERROR_CANNOTDELETE=1	;EventErrorID
Const BLITZEVENT_ERROR_CANNOTFIRE=2		;EventErrorID
Const BLITZEVENT_ERROR_CANNOTLOOP=3		;EventErrorID

;---{Public Globals}---

;BlitzEventInfo% : EventInfoInteger
;BlitzEventKind% : EventKindID

Global BlitzEventInfo%
Global BlitzEventKind%
Global BLITZEVENT_ERROR%=AddBlitzEventKind()	

;---{Public Types}---

;BlitzEventError	 : Type, dessen Handels als Info von BLITZEVENT_ERROR gefired wird,
;					   wenn BlitzEvents einen Fehler feststellt. Es kann wie jedes 
;					   andere Event abgefragt werden.

Type BlitzEventError
	Field txt$,errorid
End Type

;---{Public Functions}---

;AddBlitzEventKind% 			: Return EventKindID
;DeleteBlitzEventKind(kind%) 	: Return / 
;FireBlitzEvent(kind%,info%) 	: Return /
;NextBlitzEvent() 				: Return True, if event was fired / False, if no more events fired
;NextBlitzEventOfKind(kind%) 	: Return True, if event was fired / False, if no more events fired

Function AddBlitzEventKind%()
	Local bek.BlitzEventKind,id
	bek=Last BlitzEventKind
	If bek=Null 
		id=1 Shl 31;=kleinste negative Zahl
	Else
		id=bek\id+1
	EndIf
	bek=New BlitzEventKind
	bek\id=id
	
	Return id
End Function

Function DeleteBlitzEventKind(kind)
	Local bek.BlitzEventKind,bee.BlitzEventError
	
	If kind=BLITZEVENT_ERROR
		bee=New BlitzEventError
		bee\txt="DeleteEventKind: Eventkind [BLITZEVENT_ERROR] could not be deleted!"
		bee\errorid=BLITZEVENT_ERROR_CANNOTDELETE
		FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))
		Return 
	EndIf	
	
	bek=FindBlitzEventKind (kind)
	If bek=Null
		bee=New BlitzEventError
		bee\txt="DeleteEventKind: Eventkind ["+kind+"] could Not be deleted!"
		bee\errorid=BLITZEVENT_ERROR_CANNOTDELETE
		FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))
		Return 
	EndIf
	
	If bek\firstchild<>Null
		bee=New BlitzEventError
		bee\txt="DeleteEventKind: Eventkind ["+kind+"] is still fired!"
		bee\errorid=BLITZEVENT_ERROR_CANNOTDELETE
		FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))
		Return 
	Else
		Delete bek
	EndIf
End Function

Function FireBlitzEvent(kind%,info%)
	Local be.BlitzEvent,bek.BlitzEventKind,bee.BlitzEventError
	
	bek=FindBlitzEventKind(kind)
	If bek=Null
		bee=New BlitzEventError
		bee\txt="FireEvent: Event ["+kind+"] could not be fired!"
		bee\errorid=BLITZEVENT_ERROR_CANNOTFIRE
		FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))
		Return
	EndIf
	
	be=New BlitzEvent
	be\info=info
	be\kind=bek
	
	If bek\firstchild=Null
		bek\firstchild=be
	Else
		Insert be After bek\firstchild
	EndIf
End Function

Function NextBlitzEvent()
	Local be.BlitzEvent
	If (Last BlitzEvent)=Null
		Return False
	Else
		SetCurrentBlitzEvent (First BlitzEvent)
		Delete First BlitzEvent
		Return True
	EndIf
End Function

Function NextBlitzEventOfKind(kind)
	Local be.BlitzEvent,bek.BlitzEventKind,bee.BlitzEventError
	bek=FindBlitzEventKind(kind)
	If bek=Null
		bee=New BlitzEventError
		bee\txt="NextEventOfKind: Cannot loop through eventkind ["+kind+"] !"
		bee\errorid=BLITZEVENT_ERROR_CANNOTLOOP
		
		FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))
		Return
	EndIf
	
	If bek\firstchild=Null
		Return False
	Else
		SetCurrentBlitzEvent(bek\firstchild)
		be=bek\firstchild
		bek\firstchild=After bek\firstchild
		Delete be
		If bek\firstchild\kind<>bek;beim Verlassen des gewhlten EventKinds
			bek\firstchild=Null;Das Ende simulieren.
		EndIf
		Return True
	EndIf
End Function

;---{Internal Types}---
Type BlitzEvent
	Field info
	Field kind.BlitzEventKind
End Type

Type BlitzEventKind
	Field id
	Field firstchild.BlitzEvent
End Type

;---{Internal Functions}---
Function FindBlitzEventKind.BlitzEventKind(kind)
	Local bek.BlitzEventKind,bee.BlitzEventError
	For bek=Each BlitzEventKind
		If bek\id=kind Then Return bek
	Next
	
	bee=New BlitzEventError
	bee\txt="FindEventKind: EventKind "+kind+" not found."
	bee\errorid=BLITZEVENT_ERROR_NOTFUND
	FireBlitzEvent (BLITZEVENT_ERROR,Handle(bee))	
	
	Return Null
End Function

Function SetCurrentBlitzEvent(be.BlitzEvent)
	BlitzEventInfo=be\info
	BlitzEventKind=be\kind\id
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D